@@ -11,6 +11,7 @@ Copyright 2011, 2012 by Matthieu Boutier and Juliusz Chroboczek
 #include <sys/time.h>
 #include <sys/param.h>
 #include <time.h>
+#include <fcntl.h>
 
 #include "babeld.h"
 
@@ -91,13 +92,9 @@ kernel_route(enum babel_kernel_routes operation, const unsigned char *pref,
         case ROUTE_MODIFY:
             if(newmetric == metric && memcmp(newgate, gate, 16) == 0 &&
                newifindex == ifindex)
-                return 0;
-            debugf(BABEL_DEBUG_ROUTE, "Modify route: delete old; add new.");
-            rc = zebra_route(0, family, pref, plen, gate, ifindex, metric);
-            if (rc < 0)
-                return -1;
+		    return 0;
 
-            rc = zebra_route(1, family, pref, plen, newgate, newifindex,
+	    rc = zebra_route(1, family, pref, plen, newgate, newifindex,
                              newmetric);
             return rc;
     }
@@ -179,8 +176,7 @@ zebra_route(int add, int family, const unsigned char *pref, unsigned short plen,
     debugf(BABEL_DEBUG_ROUTE, "%s route (%s) to zebra",
            add ? "adding" : "removing",
            (family == AF_INET) ? "ipv4" : "ipv6");
-    return zclient_route_send (add ? ZEBRA_ROUTE_ADD : ZEBRA_ROUTE_DELETE,
-                               zclient, &api);
+    return zclient_route_send(add ? ZEBRA_ROUTE_ADD : ZEBRA_ROUTE_DELETE, babel_zclient, &api);
 }
 
 int
