@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - isis_redist.c
  *
  * Copyright (C) 2013-2015 Christian Franke <chris@opensourcerouting.org>
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -24,14 +11,13 @@
 #include "if.h"
 #include "linklist.h"
 #include "memory.h"
-#include "isis_memory.h"
 #include "prefix.h"
 #include "routemap.h"
 #include "stream.h"
 #include "table.h"
 #include "vty.h"
+#include "srcdest_table.h"
 
-#include "isisd/dict.h"
 #include "isisd/isis_constants.h"
 #include "isisd/isis_common.h"
 #include "isisd/isis_flags.h"
@@ -42,6 +28,11 @@
 #include "isisd/isis_route.h"
 #include "isisd/isis_zebra.h"
 
+DEFINE_MTYPE_STATIC(ISISD, ISIS_EXT_ROUTE, "ISIS redistributed route");
+DEFINE_MTYPE_STATIC(ISISD, ISIS_EXT_INFO,  "ISIS redistributed route info");
+DEFINE_MTYPE_STATIC(ISISD, ISIS_RMAP_NAME, "ISIS redistribute route-map name");
+DEFINE_MTYPE_STATIC(ISISD, ISIS_REDISTRIBUTE, "ISIS redistribute");
+
 static int redist_protocol(int family)
 {
 	if (family == AF_INET)
@@ -53,7 +44,18 @@ static int redist_protocol(int family)
 	return 0;
 }
 
-static afi_t afi_for_redist_protocol(int protocol)
+int afi_skt_for_redist_protocol(int protocol)
+{
+	if (protocol == 0)
+		return AF_INET;
+	if (protocol == 1)
+		return AF_INET6;
+
+	assert(!"Unknown redist protocol!");
+	return AF_INET;
+}
+
+afi_t afi_for_redist_protocol(int protocol)
 {
 	if (protocol == 0)
 		return AFI_IP;
@@ -71,12 +73,43 @@ static struct route_table *get_ext_info(struct isis *i, int family)
 	return i->ext_info[protocol];
 }
 
-static struct isis_redist *get_redist_settings(struct isis_area *area,
-					       int family, int type, int level)
+static struct isis_redist *isis_redist_lookup(struct isis_area *area,
+					      int family, int type, int level,
+					      uint16_t table)
 {
 	int protocol = redist_protocol(family);
+	struct listnode *node;
+	struct isis_redist *red;
 
-	return &area->redist_settings[protocol][type][level - 1];
+	if (area->redist_settings[protocol][type][level - 1]) {
+		for (ALL_LIST_ELEMENTS_RO(area->redist_settings[protocol][type]
+							       [level - 1],
+					  node, red))
+			if (red->table == table)
+				return red;
+	}
+	return NULL;
+}
+
+static struct isis_redist *isis_redist_get(struct isis_area *area, int family,
+					   int type, int level, uint16_t table)
+{
+	struct isis_redist *red;
+	int protocol;
+
+	red = isis_redist_lookup(area, family, type, level, table);
+	if (red)
+		return red;
+
+	protocol = redist_protocol(family);
+	if (area->redist_settings[protocol][type][level - 1] == NULL)
+		area->redist_settings[protocol][type][level - 1] = list_new();
+
+	red = XCALLOC(MTYPE_ISIS_REDISTRIBUTE, sizeof(struct isis_redist));
+	red->table = table;
+
+	listnode_add(area->redist_settings[protocol][type][level - 1], red);
+	return red;
 }
 
 struct route_table *get_ext_reach(struct isis_area *area, int family, int level)
@@ -86,33 +119,13 @@ struct route_table *get_ext_reach(struct isis_area *area, int family, int level)
 	return area->ext_reach[protocol][level - 1];
 }
 
-static struct route_node *
-isis_redist_route_node_create(route_table_delegate_t *delegate,
-			      struct route_table *table)
-{
-	struct route_node *node;
-	node = XCALLOC(MTYPE_ISIS_EXT_ROUTE, sizeof(*node));
-	return node;
-}
-
-static void isis_redist_route_node_destroy(route_table_delegate_t *delegate,
-					   struct route_table *table,
-					   struct route_node *node)
-{
-	if (node->info)
-		XFREE(MTYPE_ISIS_EXT_INFO, node->info);
-	XFREE(MTYPE_ISIS_EXT_ROUTE, node);
-}
-
-static route_table_delegate_t isis_redist_rt_delegate = {
-	.create_node = isis_redist_route_node_create,
-	.destroy_node = isis_redist_route_node_destroy};
-
 /* Install external reachability information into a
  * specific area for a specific level.
  * Schedule an lsp regenerate if necessary */
 static void isis_redist_install(struct isis_area *area, int level,
-				struct prefix *p, struct isis_ext_info *info)
+				const struct prefix *p,
+				const struct prefix_ipv6 *src_p,
+				struct isis_ext_info *info)
 {
 	int family = p->family;
 	struct route_table *er_table = get_ext_reach(area, family, level);
@@ -120,13 +133,12 @@ static void isis_redist_install(struct isis_area *area, int level,
 
 	if (!er_table) {
 		zlog_warn(
-			"%s: External reachability table of area %s"
-			" is not initialized.",
+			"%s: External reachability table of area %s is not initialized.",
 			__func__, area->area_tag);
 		return;
 	}
 
-	er_node = route_node_get(er_table, p);
+	er_node = srcdest_rnode_get(er_table, p, src_p);
 	if (er_node->info) {
 		route_unlock_node(er_node);
 
@@ -145,7 +157,8 @@ static void isis_redist_install(struct isis_area *area, int level,
  * specific area for a specific level.
  * Schedule an lsp regenerate if necessary. */
 static void isis_redist_uninstall(struct isis_area *area, int level,
-				  struct prefix *p)
+				  const struct prefix *p,
+				  const struct prefix_ipv6 *src_p)
 {
 	int family = p->family;
 	struct route_table *er_table = get_ext_reach(area, family, level);
@@ -153,13 +166,12 @@ static void isis_redist_uninstall(struct isis_area *area, int level,
 
 	if (!er_table) {
 		zlog_warn(
-			"%s: External reachability table of area %s"
-			" is not initialized.",
+			"%s: External reachability table of area %s is not initialized.",
 			__func__, area->area_tag);
 		return;
 	}
 
-	er_node = route_node_lookup(er_table, p);
+	er_node = srcdest_rnode_lookup(er_table, p, src_p);
 	if (!er_node)
 		return;
 	else
@@ -177,7 +189,8 @@ static void isis_redist_uninstall(struct isis_area *area, int level,
  * and prefix, using the given redistribution settings. */
 static void isis_redist_update_ext_reach(struct isis_area *area, int level,
 					 struct isis_redist *redist,
-					 struct prefix *p,
+					 const struct prefix *p,
+					 const struct prefix_ipv6 *src_p,
 					 struct isis_ext_info *info)
 {
 	struct isis_ext_info area_info;
@@ -187,8 +200,7 @@ static void isis_redist_update_ext_reach(struct isis_area *area, int level,
 	area_info.metric = redist->metric;
 
 	if (redist->map_name) {
-		map_ret =
-			route_map_apply(redist->map, p, RMAP_ISIS, &area_info);
+		map_ret = route_map_apply(redist->map, p, &area_info);
 		if (map_ret == RMAP_DENYMATCH)
 			area_info.distance = 255;
 	}
@@ -199,9 +211,9 @@ static void isis_redist_update_ext_reach(struct isis_area *area, int level,
 		area_info.distance = 255;
 
 	if (area_info.distance < 255)
-		isis_redist_install(area, level, p, &area_info);
+		isis_redist_install(area, level, p, src_p, &area_info);
 	else
-		isis_redist_uninstall(area, level, p);
+		isis_redist_uninstall(area, level, p, src_p);
 }
 
 static void isis_redist_ensure_default(struct isis *isis, int family)
@@ -222,7 +234,7 @@ static void isis_redist_ensure_default(struct isis *isis, int family)
 	} else
 		assert(!"Unknown family!");
 
-	ei_node = route_node_get(ei_table, &p);
+	ei_node = srcdest_rnode_get(ei_table, &p, NULL);
 	if (ei_node->info) {
 		route_unlock_node(ei_node);
 		return;
@@ -237,9 +249,57 @@ static void isis_redist_ensure_default(struct isis *isis, int family)
 	info->metric = MAX_WIDE_PATH_METRIC;
 }
 
+static int _isis_redist_table_is_present(const struct lyd_node *dnode, void *arg)
+{
+	struct isis_redist_table_present_args *rtda = arg;
+
+	/* This entry is the caller, so skip it. */
+	if (yang_dnode_get_uint16(dnode, "table") !=
+	    (uint16_t)atoi(rtda->rtda_table))
+		return YANG_ITER_CONTINUE;
+
+	/* found */
+	rtda->rtda_found = true;
+	return YANG_ITER_CONTINUE;
+}
+
+static int _isis_redist_table_get_first_cb(const struct lyd_node *dnode,
+					   void *arg)
+{
+	uint16_t *table = arg;
+
+	*table = yang_dnode_get_uint16(dnode, "table");
+	return YANG_ITER_STOP;
+}
+
+uint16_t isis_redist_table_get_first(const struct vty *vty,
+				     struct isis_redist_table_present_args *rtda)
+{
+	uint16_t table = 0;
+
+	yang_dnode_iterate(_isis_redist_table_get_first_cb, &table,
+			   vty->candidate_config->dnode,
+			   "%s/redistribute/%s[protocol='table'][level='%s']/table",
+			   VTY_CURR_XPATH, rtda->rtda_ip, rtda->rtda_level);
+	return table;
+}
+
+bool isis_redist_table_is_present(const struct vty *vty,
+				  struct isis_redist_table_present_args *rtda)
+{
+	rtda->rtda_found = false;
+	yang_dnode_iterate(_isis_redist_table_is_present, rtda,
+			   vty->candidate_config->dnode,
+			   "%s/redistribute/%s[protocol='table'][level='%s']/table",
+			   VTY_CURR_XPATH, rtda->rtda_ip, rtda->rtda_level);
+
+	return rtda->rtda_found;
+}
+
 /* Handle notification about route being added */
-void isis_redist_add(int type, struct prefix *p, uint8_t distance,
-		     uint32_t metric)
+void isis_redist_add(struct isis *isis, int type, struct prefix *p,
+		     struct prefix_ipv6 *src_p, uint8_t distance,
+		     uint32_t metric, const route_tag_t tag, uint16_t table)
 {
 	int family = p->family;
 	struct route_table *ei_table = get_ext_info(isis, family);
@@ -250,11 +310,8 @@ void isis_redist_add(int type, struct prefix *p, uint8_t distance,
 	int level;
 	struct isis_redist *redist;
 
-	char debug_buf[BUFSIZ];
-	prefix2str(p, debug_buf, sizeof(debug_buf));
-
-	zlog_debug("%s: New route %s from %s: distance %d.", __func__,
-		   debug_buf, zebra_route_string(type), distance);
+	zlog_debug("%s: New route %pFX from %s: distance %d.", __func__, p,
+		   zebra_route_string(type), distance);
 
 	if (!ei_table) {
 		zlog_warn("%s: External information table not initialized.",
@@ -262,7 +319,7 @@ void isis_redist_add(int type, struct prefix *p, uint8_t distance,
 		return;
 	}
 
-	ei_node = route_node_get(ei_table, p);
+	ei_node = srcdest_rnode_get(ei_table, p, src_p);
 	if (ei_node->info)
 		route_unlock_node(ei_node);
 	else
@@ -273,22 +330,27 @@ void isis_redist_add(int type, struct prefix *p, uint8_t distance,
 	info->origin = type;
 	info->distance = distance;
 	info->metric = metric;
+	info->tag = tag;
 
-	if (is_default_prefix(p))
+	if (is_default_prefix(p)
+	    && (!src_p || !src_p->prefixlen)) {
 		type = DEFAULT_ROUTE;
+	}
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area))
 		for (level = 1; level <= ISIS_LEVELS; level++) {
-			redist = get_redist_settings(area, family, type, level);
-			if (!redist->redist)
+			redist = isis_redist_lookup(area, family, type, level,
+						    table);
+			if (!redist || !redist->redist)
 				continue;
 
 			isis_redist_update_ext_reach(area, level, redist, p,
-						     info);
+						     src_p, info);
 		}
 }
 
-void isis_redist_delete(int type, struct prefix *p)
+void isis_redist_delete(struct isis *isis, int type, struct prefix *p,
+			struct prefix_ipv6 *src_p, uint16_t table)
 {
 	int family = p->family;
 	struct route_table *ei_table = get_ext_info(isis, family);
@@ -298,18 +360,17 @@ void isis_redist_delete(int type, struct prefix *p)
 	int level;
 	struct isis_redist *redist;
 
-	char debug_buf[BUFSIZ];
-	prefix2str(p, debug_buf, sizeof(debug_buf));
-
-	zlog_debug("%s: Removing route %s from %s.", __func__, debug_buf,
+	zlog_debug("%s: Removing route %pFX from %s.", __func__, p,
 		   zebra_route_string(type));
 
-	if (is_default_prefix(p)) {
+	if (is_default_prefix(p)
+	    && (!src_p || !src_p->prefixlen)) {
 		/* Don't remove default route but add synthetic route for use
 		 * by "default-information originate always". Areas without the
 		 * "always" setting will ignore routes with origin
 		 * DEFAULT_ROUTE. */
-		isis_redist_add(DEFAULT_ROUTE, p, 254, MAX_WIDE_PATH_METRIC);
+		isis_redist_add(isis, DEFAULT_ROUTE, p, NULL, 254,
+				MAX_WIDE_PATH_METRIC, 0, table);
 		return;
 	}
 
@@ -319,14 +380,11 @@ void isis_redist_delete(int type, struct prefix *p)
 		return;
 	}
 
-	ei_node = route_node_lookup(ei_table, p);
+	ei_node = srcdest_rnode_lookup(ei_table, p, src_p);
 	if (!ei_node || !ei_node->info) {
-		char buf[BUFSIZ];
-		prefix2str(p, buf, sizeof(buf));
 		zlog_warn(
-			"%s: Got a delete for %s route %s, but that route"
-			" was never added.",
-			__func__, zebra_route_string(type), buf);
+			"%s: Got a delete for %s route %pFX, but that route was never added.",
+			__func__, zebra_route_string(type), p);
 		if (ei_node)
 			route_unlock_node(ei_node);
 		return;
@@ -334,12 +392,13 @@ void isis_redist_delete(int type, struct prefix *p)
 	route_unlock_node(ei_node);
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area))
-		for (level = 1; level < ISIS_LEVELS; level++) {
-			redist = get_redist_settings(area, family, type, level);
-			if (!redist->redist)
+		for (level = ISIS_LEVEL1; level <= ISIS_LEVEL2; level++) {
+			redist = isis_redist_lookup(area, family, type, level,
+						    table);
+			if (!redist || !redist->redist)
 				continue;
 
-			isis_redist_uninstall(area, level, p);
+			isis_redist_uninstall(area, level, p, src_p);
 		}
 
 	XFREE(MTYPE_ISIS_EXT_INFO, ei_node->info);
@@ -350,64 +409,80 @@ static void isis_redist_routemap_set(struct isis_redist *redist,
 				     const char *routemap)
 {
 	if (redist->map_name) {
-		XFREE(MTYPE_ISIS, redist->map_name);
+		XFREE(MTYPE_ISIS_RMAP_NAME, redist->map_name);
+		route_map_counter_decrement(redist->map);
 		redist->map = NULL;
 	}
 
 	if (routemap && strlen(routemap)) {
-		redist->map_name = XSTRDUP(MTYPE_ISIS, routemap);
+		redist->map_name = XSTRDUP(MTYPE_ISIS_RMAP_NAME, routemap);
 		redist->map = route_map_lookup_by_name(routemap);
+		route_map_counter_increment(redist->map);
 	}
 }
 
-static void isis_redist_update_zebra_subscriptions(struct isis *isis)
+void isis_redist_free(struct isis *isis)
 {
-	struct listnode *node;
-	struct isis_area *area;
-	int type;
-	int level;
-	int protocol;
+	struct route_node *rn;
+	int i;
 
-	char do_subscribe[REDIST_PROTOCOL_COUNT][ZEBRA_ROUTE_MAX + 1];
+	for (i = 0; i < REDIST_PROTOCOL_COUNT; i++) {
+		if (!isis->ext_info[i])
+			continue;
 
-	memset(do_subscribe, 0, sizeof(do_subscribe));
-
-	for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area))
-		for (protocol = 0; protocol < REDIST_PROTOCOL_COUNT; protocol++)
-			for (type = 0; type < ZEBRA_ROUTE_MAX + 1; type++)
-				for (level = 0; level < ISIS_LEVELS; level++)
-					if (area->redist_settings[protocol]
-								 [type]
-								 [level].redist)
-						do_subscribe[protocol][type] =
-							1;
-
-	for (protocol = 0; protocol < REDIST_PROTOCOL_COUNT; protocol++)
-		for (type = 0; type < ZEBRA_ROUTE_MAX + 1; type++) {
-			/* This field is actually controlling transmission of
-			 * the IS-IS
-			 * routes to Zebra and has nothing to do with
-			 * redistribution,
-			 * so skip it. */
-			if (type == ZEBRA_ROUTE_ISIS)
-				continue;
-
-			afi_t afi = afi_for_redist_protocol(protocol);
-
-			if (do_subscribe[protocol][type])
-				isis_zebra_redistribute_set(afi, type);
-			else
-				isis_zebra_redistribute_unset(afi, type);
+		for (rn = route_top(isis->ext_info[i]); rn;
+		     rn = srcdest_route_next(rn)) {
+			if (rn->info)
+				XFREE(MTYPE_ISIS_EXT_INFO, rn->info);
 		}
+
+		route_table_finish(isis->ext_info[i]);
+		isis->ext_info[i] = NULL;
+	}
 }
 
-static void isis_redist_set(struct isis_area *area, int level, int family,
-			    int type, uint32_t metric, const char *routemap,
-			    int originate_type)
+void isis_redist_update(struct isis_area *area, int level, int family, int type, uint16_t table)
+{
+	struct isis_redist *redist;
+	struct route_table *ei_table;
+	struct route_node *rn;
+	struct isis_ext_info *info;
+
+	redist = isis_redist_lookup(area, family, type, level, table);
+	if (!redist)
+		return;
+
+	ei_table = get_ext_info(area->isis, family);
+	for (rn = route_top(ei_table); rn; rn = srcdest_route_next(rn)) {
+		if (!rn->info)
+			continue;
+		info = rn->info;
+
+		const struct prefix *p, *src_p;
+
+		srcdest_rnode_prefixes(rn, &p, &src_p);
+
+		if (type == DEFAULT_ROUTE) {
+			if (!is_default_prefix(p) || (src_p && src_p->prefixlen)) {
+				continue;
+			}
+		} else {
+			if (info->origin != type)
+				continue;
+		}
+
+		isis_redist_update_ext_reach(area, level, redist, p,
+					     (const struct prefix_ipv6 *)src_p, info);
+	}
+}
+
+void isis_redist_set(struct isis_area *area, int level, int family, int type,
+		     uint32_t metric, const char *routemap, int originate_type,
+		     uint16_t table)
 {
 	int protocol = redist_protocol(family);
-	struct isis_redist *redist =
-		get_redist_settings(area, family, type, level);
+	struct isis_redist *redist = isis_redist_get(area, family, type, level,
+						     table);
 	int i;
 	struct route_table *ei_table;
 	struct route_node *rn;
@@ -418,68 +493,86 @@ static void isis_redist_set(struct isis_area *area, int level, int family,
 	isis_redist_routemap_set(redist, routemap);
 
 	if (!area->ext_reach[protocol][level - 1]) {
-		area->ext_reach[protocol][level - 1] =
-			route_table_init_with_delegate(
-				&isis_redist_rt_delegate);
+		area->ext_reach[protocol][level - 1] = srcdest_table_init();
 	}
 
-	for (i = 0; i < REDIST_PROTOCOL_COUNT; i++)
+	for (i = 0; i < REDIST_PROTOCOL_COUNT; i++) {
 		if (!area->isis->ext_info[i]) {
-			area->isis->ext_info[i] =
-				route_table_init_with_delegate(
-					&isis_redist_rt_delegate);
+			area->isis->ext_info[i] = srcdest_table_init();
 		}
+	}
 
-	isis_redist_update_zebra_subscriptions(area->isis);
+	isis_zebra_redistribute_set(afi_for_redist_protocol(protocol), type,
+				    area->isis->vrf_id, redist->table);
 
 	if (type == DEFAULT_ROUTE && originate_type == DEFAULT_ORIGINATE_ALWAYS)
 		isis_redist_ensure_default(area->isis, family);
 
 	ei_table = get_ext_info(area->isis, family);
-	for (rn = route_top(ei_table); rn; rn = route_next(rn)) {
+	for (rn = route_top(ei_table); rn; rn = srcdest_route_next(rn)) {
 		if (!rn->info)
 			continue;
 		info = rn->info;
 
+		const struct prefix *p, *src_p;
+
+		srcdest_rnode_prefixes(rn, &p, &src_p);
+
 		if (type == DEFAULT_ROUTE) {
-			if (!is_default_prefix(&rn->p))
+			if (!is_default_prefix(p)
+			    || (src_p && src_p->prefixlen)) {
 				continue;
+			}
 		} else {
 			if (info->origin != type)
 				continue;
 		}
 
-		isis_redist_update_ext_reach(area, level, redist, &rn->p, info);
+		isis_redist_update_ext_reach(area, level, redist, p,
+					     (const struct prefix_ipv6 *)src_p,
+					     info);
 	}
 }
 
-static void isis_redist_unset(struct isis_area *area, int level, int family,
-			      int type)
+void isis_redist_unset(struct isis_area *area, int level, int family, int type,
+		       uint16_t table)
 {
-	struct isis_redist *redist =
-		get_redist_settings(area, family, type, level);
+	struct isis_redist *redist = isis_redist_lookup(area, family, type,
+							level, table);
 	struct route_table *er_table = get_ext_reach(area, family, level);
 	struct route_node *rn;
 	struct isis_ext_info *info;
+	struct list *redist_list;
+	int protocol = redist_protocol(family);
 
-	if (!redist->redist)
+	if (!redist || !redist->redist)
 		return;
 
 	redist->redist = 0;
+
+	redist_list = area->redist_settings[protocol][type][level - 1];
+	listnode_delete(redist_list, redist);
+	XFREE(MTYPE_ISIS_REDISTRIBUTE, redist);
+
 	if (!er_table) {
 		zlog_warn("%s: External reachability table uninitialized.",
 			  __func__);
 		return;
 	}
 
-	for (rn = route_top(er_table); rn; rn = route_next(rn)) {
+	for (rn = route_top(er_table); rn; rn = srcdest_route_next(rn)) {
 		if (!rn->info)
 			continue;
 		info = rn->info;
 
+		const struct prefix *p, *src_p;
+		srcdest_rnode_prefixes(rn, &p, &src_p);
+
 		if (type == DEFAULT_ROUTE) {
-			if (!is_default_prefix(&rn->p))
+			if (!is_default_prefix(p)
+			    || (src_p && src_p->prefixlen)) {
 				continue;
+			}
 		} else {
 			if (info->origin != type)
 				continue;
@@ -490,41 +583,63 @@ static void isis_redist_unset(struct isis_area *area, int level, int family,
 	}
 
 	lsp_regenerate_schedule(area, level, 0);
-	isis_redist_update_zebra_subscriptions(area->isis);
+	isis_zebra_redistribute_unset(afi_for_redist_protocol(protocol), type,
+				      area->isis->vrf_id, table);
 }
 
 void isis_redist_area_finish(struct isis_area *area)
 {
+	struct route_node *rn;
 	int protocol;
 	int level;
 	int type;
+	struct isis_redist *redist;
+	struct listnode *node, *nnode;
+	struct list *redist_list;
 
 	for (protocol = 0; protocol < REDIST_PROTOCOL_COUNT; protocol++)
 		for (level = 0; level < ISIS_LEVELS; level++) {
 			for (type = 0; type < ZEBRA_ROUTE_MAX + 1; type++) {
-				struct isis_redist *redist;
-
-				redist = &area->redist_settings[protocol][type]
-							       [level];
-				redist->redist = 0;
-				if (redist->map_name)
-					XFREE(MTYPE_ISIS, redist->map_name);
+				redist_list = area->redist_settings[protocol]
+								   [type][level];
+				if (!redist_list)
+					continue;
+				for (ALL_LIST_ELEMENTS(redist_list, node, nnode,
+						       redist)) {
+					redist->redist = 0;
+					XFREE(MTYPE_ISIS_RMAP_NAME,
+					      redist->map_name);
+					isis_zebra_redistribute_unset(
+						afi_for_redist_protocol(protocol),
+						type, area->isis->vrf_id,
+						redist->table);
+					listnode_delete(redist_list, redist);
+					XFREE(MTYPE_ISIS_REDISTRIBUTE, redist);
+				}
+				list_delete(&redist_list);
+			}
+			if (!area->ext_reach[protocol][level])
+				continue;
+			for (rn = route_top(area->ext_reach[protocol][level]);
+			     rn; rn = srcdest_route_next(rn)) {
+				if (rn->info)
+					XFREE(MTYPE_ISIS_EXT_INFO, rn->info);
 			}
 			route_table_finish(area->ext_reach[protocol][level]);
+			area->ext_reach[protocol][level] = NULL;
 		}
-
-	isis_redist_update_zebra_subscriptions(area->isis);
 }
 
+#ifdef FABRICD
 DEFUN (isis_redistribute,
        isis_redistribute_cmd,
-       "redistribute <ipv4|ipv6> " FRR_REDIST_STR_ISISD " <level-1|level-2> [<metric (0-16777215)|route-map WORD>]",
+       "redistribute <ipv4 " PROTO_IP_REDIST_STR "|ipv6 " PROTO_IP6_REDIST_STR ">"
+       " [{metric (0-16777215)|route-map RMAP_NAME}]",
        REDIST_STR
        "Redistribute IPv4 routes\n"
+       PROTO_IP_REDIST_HELP
        "Redistribute IPv6 routes\n"
-       FRR_REDIST_HELP_STR_ISISD
-       "Redistribute into level-1\n"
-       "Redistribute into level-2\n"
+       PROTO_IP6_REDIST_HELP
        "Metric for redistributed routes\n"
        "ISIS default metric\n"
        "Route map reference\n"
@@ -532,8 +647,7 @@ DEFUN (isis_redistribute,
 {
 	int idx_afi = 1;
 	int idx_protocol = 2;
-	int idx_level = 3;
-	int idx_metric_rmap = 4;
+	int idx_metric_rmap = 1;
 	VTY_DECLVAR_CONTEXT(isis_area, area);
 	int family;
 	int afi;
@@ -554,52 +668,38 @@ DEFUN (isis_redistribute,
 	if (type < 0)
 		return CMD_WARNING_CONFIG_FAILED;
 
-	if (!strcmp("level-1", argv[idx_level]->arg))
-		level = 1;
-	else if (!strcmp("level-2", argv[idx_level]->arg))
-		level = 2;
-	else
-		return CMD_WARNING_CONFIG_FAILED;
+	level = 2;
 
 	if ((area->is_type & level) != level) {
 		vty_out(vty, "Node is not a level-%d IS\n", level);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
-	if (argc > idx_metric_rmap + 1) {
-		if (argv[idx_metric_rmap + 1]->arg[0] == '\0')
-			return CMD_WARNING_CONFIG_FAILED;
-
-		if (strmatch(argv[idx_metric_rmap]->text, "metric")) {
-			char *endp;
-			metric = strtoul(argv[idx_metric_rmap + 1]->arg, &endp,
-					 10);
-
-			if (*endp != '\0')
-				return CMD_WARNING_CONFIG_FAILED;
-		} else {
-			routemap = argv[idx_metric_rmap + 1]->arg;
-		}
+	if (argv_find(argv, argc, "metric", &idx_metric_rmap)) {
+		metric = strtoul(argv[idx_metric_rmap + 1]->arg, NULL, 10);
 	}
 
-	isis_redist_set(area, level, family, type, metric, routemap, 0);
+	idx_metric_rmap = 1;
+	if (argv_find(argv, argc, "route-map", &idx_metric_rmap)) {
+		routemap = argv[idx_metric_rmap + 1]->arg;
+	}
+
+	isis_redist_set(area, level, family, type, metric, routemap, 0, 0);
 	return 0;
 }
 
 DEFUN (no_isis_redistribute,
        no_isis_redistribute_cmd,
-       "no redistribute <ipv4|ipv6> " FRR_REDIST_STR_ISISD " <level-1|level-2>",
+       "no redistribute <ipv4 " PROTO_IP_REDIST_STR "|ipv6 " PROTO_IP6_REDIST_STR ">",
        NO_STR
        REDIST_STR
        "Redistribute IPv4 routes\n"
+       PROTO_IP_REDIST_HELP
        "Redistribute IPv6 routes\n"
-       FRR_REDIST_HELP_STR_ISISD
-       "Redistribute into level-1\n"
-       "Redistribute into level-2\n")
+       PROTO_IP6_REDIST_HELP)
 {
 	int idx_afi = 2;
 	int idx_protocol = 3;
-	int idx_level = 4;
 	VTY_DECLVAR_CONTEXT(isis_area, area);
 	int type;
 	int level;
@@ -618,21 +718,19 @@ DEFUN (no_isis_redistribute,
 	if (type < 0)
 		return CMD_WARNING_CONFIG_FAILED;
 
-	level = strmatch("level-1", argv[idx_level]->text) ? 1 : 2;
+	level = 2;
 
-	isis_redist_unset(area, level, family, type);
+	isis_redist_unset(area, level, family, type, 0);
 	return 0;
 }
 
 DEFUN (isis_default_originate,
        isis_default_originate_cmd,
-       "default-information originate <ipv4|ipv6> <level-1|level-2> [always] [<metric (0-16777215)|route-map WORD>]",
+       "default-information originate <ipv4|ipv6> [always] [{metric (0-16777215)|route-map RMAP_NAME}]",
        "Control distribution of default information\n"
        "Distribute a default route\n"
        "Distribute default route for IPv4\n"
        "Distribute default route for IPv6\n"
-       "Distribute default route into level-1\n"
-       "Distribute default route into level-2\n"
        "Always advertise default route\n"
        "Metric for default route\n"
        "ISIS default metric\n"
@@ -640,9 +738,8 @@ DEFUN (isis_default_originate,
        "Pointer to route-map entries\n")
 {
 	int idx_afi = 2;
-	int idx_level = 3;
-	int idx_always = 4;
-	int idx_metric_rmap = 4;
+	int idx_always = fabricd ? 3 : 4;
+	int idx_metric_rmap = 1;
 	VTY_DECLVAR_CONTEXT(isis_area, area);
 	int family;
 	int originate_type = DEFAULT_ORIGINATE;
@@ -654,7 +751,7 @@ DEFUN (isis_default_originate,
 	if (family < 0)
 		return CMD_WARNING_CONFIG_FAILED;
 
-	level = strmatch("level-1", argv[idx_level]->text) ? 1 : 2;
+	level = 2;
 
 	if ((area->is_type & level) != level) {
 		vty_out(vty, "Node is not a level-%d IS\n", level);
@@ -666,12 +763,13 @@ DEFUN (isis_default_originate,
 		idx_metric_rmap++;
 	}
 
-	if (argc > idx_metric_rmap) {
-		if (strmatch(argv[idx_metric_rmap]->text, "metric"))
-			metric = strtoul(argv[idx_metric_rmap + 1]->arg, NULL,
-					 10);
-		else
-			routemap = argv[idx_metric_rmap + 1]->arg;
+	if (argv_find(argv, argc, "metric", &idx_metric_rmap)) {
+		metric = strtoul(argv[idx_metric_rmap + 1]->arg, NULL, 10);
+	}
+
+	idx_metric_rmap = 1;
+	if (argv_find(argv, argc, "route-map", &idx_metric_rmap)) {
+		routemap = argv[idx_metric_rmap + 1]->arg;
 	}
 
 	if (family == AF_INET6 && originate_type != DEFAULT_ORIGINATE_ALWAYS) {
@@ -682,23 +780,20 @@ DEFUN (isis_default_originate,
 	}
 
 	isis_redist_set(area, level, family, DEFAULT_ROUTE, metric, routemap,
-			originate_type);
+			originate_type, 0);
 	return 0;
 }
 
 DEFUN (no_isis_default_originate,
        no_isis_default_originate_cmd,
-       "no default-information originate <ipv4|ipv6> <level-1|level-2>",
+       "no default-information originate <ipv4|ipv6>",
        NO_STR
        "Control distribution of default information\n"
        "Distribute a default route\n"
        "Distribute default route for IPv4\n"
-       "Distribute default route for IPv6\n"
-       "Distribute default route into level-1\n"
-       "Distribute default route into level-2\n")
+       "Distribute default route for IPv6\n")
 {
 	int idx_afi = 3;
-	int idx_level = 4;
 	VTY_DECLVAR_CONTEXT(isis_area, area);
 	int family;
 	int level;
@@ -707,16 +802,12 @@ DEFUN (no_isis_default_originate,
 	if (family < 0)
 		return CMD_WARNING_CONFIG_FAILED;
 
-	if (strmatch("level-1", argv[idx_level]->text))
-		level = 1;
-	else if (strmatch("level-2", argv[idx_level]->text))
-		level = 2;
-	else
-		return CMD_WARNING_CONFIG_FAILED;
+	level = 2;
 
-	isis_redist_unset(area, level, family, DEFAULT_ROUTE);
+	isis_redist_unset(area, level, family, DEFAULT_ROUTE, 0);
 	return 0;
 }
+#endif /* ifdef FABRICD */
 
 int isis_redist_config_write(struct vty *vty, struct isis_area *area,
 			     int family)
@@ -725,7 +816,9 @@ int isis_redist_config_write(struct vty *vty, struct isis_area *area,
 	int level;
 	int write = 0;
 	struct isis_redist *redist;
+	struct list *redist_list;
 	const char *family_str;
+	struct listnode *node;
 
 	if (family == AF_INET)
 		family_str = "ipv4";
@@ -735,31 +828,46 @@ int isis_redist_config_write(struct vty *vty, struct isis_area *area,
 		return 0;
 
 	for (type = 0; type < ZEBRA_ROUTE_MAX; type++) {
-		if (type == ZEBRA_ROUTE_ISIS)
+		if (type == PROTO_TYPE)
 			continue;
 
 		for (level = 1; level <= ISIS_LEVELS; level++) {
-			redist = get_redist_settings(area, family, type, level);
-			if (!redist->redist)
+			redist_list = area->redist_settings[redist_protocol(
+				family)][type][level - 1];
+			if (!redist_list)
 				continue;
-			vty_out(vty, " redistribute %s %s level-%d", family_str,
-				zebra_route_string(type), level);
-			if (redist->metric)
-				vty_out(vty, " metric %u", redist->metric);
-			if (redist->map_name)
-				vty_out(vty, " route-map %s", redist->map_name);
-			vty_out(vty, "\n");
-			write++;
+			for (ALL_LIST_ELEMENTS_RO(redist_list, node, redist)) {
+				if (!redist->redist)
+					continue;
+				vty_out(vty, " redistribute %s %s", family_str,
+					zebra_route_string(type));
+				if (type == ZEBRA_ROUTE_TABLE)
+					vty_out(vty, " %u", redist->table);
+				if (!fabricd)
+					vty_out(vty, " level-%d", level);
+				if (redist->metric)
+					vty_out(vty, " metric %u",
+						redist->metric);
+				if (redist->map_name)
+					vty_out(vty, " route-map %s",
+						redist->map_name);
+				vty_out(vty, "\n");
+				write++;
+			}
 		}
 	}
 
 	for (level = 1; level <= ISIS_LEVELS; level++) {
-		redist =
-			get_redist_settings(area, family, DEFAULT_ROUTE, level);
+		redist = isis_redist_lookup(area, family, DEFAULT_ROUTE, level,
+					    0);
+		if (!redist)
+			continue;
 		if (!redist->redist)
 			continue;
-		vty_out(vty, " default-information originate %s level-%d",
-			family_str, level);
+		vty_out(vty, " default-information originate %s",
+			family_str);
+		if (!fabricd)
+			vty_out(vty, " level-%d", level);
 		if (redist->redist == DEFAULT_ORIGINATE_ALWAYS)
 			vty_out(vty, " always");
 		if (redist->metric)
@@ -775,8 +883,11 @@ int isis_redist_config_write(struct vty *vty, struct isis_area *area,
 
 void isis_redist_init(void)
 {
-	install_element(ISIS_NODE, &isis_redistribute_cmd);
-	install_element(ISIS_NODE, &no_isis_redistribute_cmd);
-	install_element(ISIS_NODE, &isis_default_originate_cmd);
-	install_element(ISIS_NODE, &no_isis_default_originate_cmd);
+#ifdef FABRICD
+	install_element(ROUTER_NODE, &isis_redistribute_cmd);
+	install_element(ROUTER_NODE, &no_isis_redistribute_cmd);
+
+	install_element(ROUTER_NODE, &isis_default_originate_cmd);
+	install_element(ROUTER_NODE, &no_isis_default_originate_cmd);
+#endif /* ifdef FABRICD */
 }
