@@ -14,9 +14,17 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#ifndef HAVE_LIBCRYPT
+#ifdef HAVE_LIBCRYPTO
+#include <openssl/des.h>
+#define crypt DES_crypt
+#endif
+#endif
+
 #include <lib/version.h>
 
 #include "command.h"
+#include "debug.h"
 #include "frrstr.h"
 #include "memory.h"
 #include "log.h"
@@ -40,6 +48,7 @@
 #include "northbound_cli.h"
 #include "network.h"
 #include "routemap.h"
+#include "frregex_real.h"
 
 #include "frrscript.h"
 
@@ -1453,7 +1462,7 @@ DEFUN (config_help,
        "Description of the interactive help system\n")
 {
 	vty_out(vty,
-		"Quagga VTY provides advanced help feature.  When you need help,\n\
+		"FRR VTY provides advanced help feature.  When you need help,\n\
 anytime at the command line please press '?'.\n\
 \n\
 If nothing matches, the help list will be empty and you must backup\n\
@@ -2316,21 +2325,6 @@ DEFUN (no_banner_motd,
 	return CMD_SUCCESS;
 }
 
-DEFUN(allow_reserved_ranges, allow_reserved_ranges_cmd, "allow-reserved-ranges",
-      "Allow using IPv4 (Class E) reserved IP space\n")
-{
-	host.allow_reserved_ranges = true;
-	return CMD_SUCCESS;
-}
-
-DEFUN(no_allow_reserved_ranges, no_allow_reserved_ranges_cmd,
-      "no allow-reserved-ranges",
-      NO_STR "Allow using IPv4 (Class E) reserved IP space\n")
-{
-	host.allow_reserved_ranges = false;
-	return CMD_SUCCESS;
-}
-
 int cmd_find_cmds(struct vty *vty, struct cmd_token **argv, int argc)
 {
 	const struct cmd_node *node;
@@ -2463,8 +2457,7 @@ const char *host_config_get(void)
 void cmd_show_lib_debugs(struct vty *vty)
 {
 	route_map_show_debug(vty);
-	mgmt_debug_be_client_show_debug(vty);
-	mgmt_debug_fe_client_show_debug(vty);
+	debug_status_write(vty);
 }
 
 void install_default(enum node_type node)
@@ -2597,8 +2590,6 @@ void cmd_init(int terminal)
 		install_element(CONFIG_NODE, &no_banner_motd_cmd);
 		install_element(CONFIG_NODE, &service_terminal_length_cmd);
 		install_element(CONFIG_NODE, &no_service_terminal_length_cmd);
-		install_element(CONFIG_NODE, &allow_reserved_ranges_cmd);
-		install_element(CONFIG_NODE, &no_allow_reserved_ranges_cmd);
 
 		log_cmd_init();
 		vrf_install_commands();
