@@ -42,6 +42,15 @@ static void work_queue_item_free(struct work_queue_item *item)
 	return;
 }
 
+static inline void work_queue_item_dequeue(struct work_queue *wq,
+					   struct work_queue_item *item)
+{
+	assert(wq->item_count > 0);
+
+	wq->item_count--;
+	STAILQ_REMOVE(&wq->items, item, work_queue_item, wq);
+}
+
 static void work_queue_item_remove(struct work_queue *wq,
 				   struct work_queue_item *item)
 {
@@ -87,7 +96,7 @@ void work_queue_free_and_null(struct work_queue **wqp)
 {
 	struct work_queue *wq = *wqp;
 
-	EVENT_OFF(wq->thread);
+	event_cancel(&wq->thread);
 
 	while (!work_queue_empty(wq)) {
 		struct work_queue_item *item = work_queue_last_item(wq);
@@ -133,6 +142,13 @@ static int work_queue_schedule(struct work_queue *wq, unsigned int delay)
 		return 0;
 }
 
+static inline void work_queue_item_enqueue(struct work_queue *wq,
+					   struct work_queue_item *item)
+{
+	STAILQ_INSERT_TAIL(&wq->items, item, wq);
+	wq->item_count++;
+}
+
 void work_queue_add(struct work_queue *wq, void *data)
 {
 	struct work_queue_item *item;
@@ -197,7 +213,7 @@ void workqueue_cmd_init(void)
  */
 void work_queue_plug(struct work_queue *wq)
 {
-	EVENT_OFF(wq->thread);
+	event_cancel(&wq->thread);
 
 	UNSET_FLAG(wq->flags, WQ_UNPLUGGED);
 }
@@ -265,17 +281,14 @@ void work_queue_run(struct event *thread)
 		do {
 			ret = wq->spec.workfunc(wq, item->data);
 			item->ran++;
-		} while ((ret == WQ_RETRY_NOW)
-			 && (item->ran < wq->spec.max_retries));
+		} while (item->ran < wq->spec.max_retries);
 
 		switch (ret) {
 		case WQ_QUEUE_BLOCKED: {
 			/* decrement item->ran again, cause this isn't an item
-			 * specific error, and fall through to WQ_RETRY_LATER
+			 * specific error, and retry later
 			 */
 			item->ran--;
-		}
-		case WQ_RETRY_LATER: {
 			goto stats;
 		}
 		case WQ_REQUEUE: {
@@ -295,10 +308,6 @@ void work_queue_run(struct event *thread)
 				titem = item;
 			break;
 		}
-		case WQ_RETRY_NOW:
-		/* a RETRY_NOW that gets here has exceeded max_tries, same as
-		 * ERROR */
-		/* fallthru */
 		case WQ_SUCCESS:
 		default: {
 			work_queue_item_remove(wq, item);
@@ -350,8 +359,7 @@ stats:
 
 	/* Is the queue done yet? If it is, call the completion callback. */
 	if (!work_queue_empty(wq)) {
-		if (ret == WQ_RETRY_LATER ||
-		    ret == WQ_QUEUE_BLOCKED)
+		if (ret == WQ_QUEUE_BLOCKED)
 			work_queue_schedule(wq, wq->spec.retry);
 		else
 			work_queue_schedule(wq, 0);
