@@ -44,5 +44,7 @@ DECLARE_MTYPE(OSPF_GR_HELPER);
 DECLARE_MTYPE(OSPF_EXTERNAL_RT_AGGR);
 DECLARE_MTYPE(OSPF_P_SPACE);
 DECLARE_MTYPE(OSPF_Q_SPACE);
+DECLARE_MTYPE(OSPF_LSA_LIST);
+DECLARE_MTYPE(OSPF_LSDB_NODE);
 
 #endif /* _QUAGGA_OSPF_MEMORY_H */
