@@ -213,7 +213,7 @@ DEFPY (install_routes,
        "The vrf we would like to install into if non-default\n"
        "The NAME of the vrf\n"
        "v4 Address to start /32 generation at\n"
-       "v6 Address to start /32 generation at\n"
+       "v6 Address to start /128 generation at\n"
        "Nexthop to use(Can be an IPv4 or IPv6 address)\n"
        "V4 Nexthop address to use\n"
        "V6 Nexthop address to use\n"
@@ -372,7 +372,7 @@ DEFPY (install_seg6_routes,
        "The vrf we would like to install into if non-default\n"
        "The NAME of the vrf\n"
        "v4 Address to start /32 generation at\n"
-       "v6 Address to start /32 generation at\n"
+       "v6 Address to start /128 generation at\n"
        "Nexthop-seg6 to use\n"
        "V6 Nexthop address to use\n"
        "Encap mode\n"
@@ -429,7 +429,7 @@ DEFPY (install_seg6_routes,
 	sg.r.nhop.gate.ipv6 = seg6_nh6;
 	sg.r.nhop.vrf_id = vrf->vrf_id;
 	sg.r.nhop_group.nexthop = &sg.r.nhop;
-	nexthop_add_srv6_seg6(&sg.r.nhop, &seg6_seg, 1);
+	nexthop_add_srv6_seg6(&sg.r.nhop, &seg6_seg, 1, SRV6_HEADEND_BEHAVIOR_H_ENCAPS);
 
 	sg.r.vrf_id = vrf->vrf_id;
 	sharp_install_routes_helper(&prefix, sg.r.vrf_id, sg.r.inst, 0,
@@ -439,51 +439,44 @@ DEFPY (install_seg6_routes,
 	return CMD_SUCCESS;
 }
 
-DEFPY (install_seg6local_routes,
-       install_seg6local_routes_cmd,
-       "sharp install seg6local-routes [vrf NAME$vrf_name]\
+DEFPY(install_seg6local_segs_routes, install_seg6local_segs_routes_cmd,
+      "sharp install seg6local-routes [vrf NAME$vrf_name]\
 	  X:X::X:X$start6\
 	  nexthop-seg6local NAME$seg6l_oif\
-	     <End$seg6l_end|\
-	      End_X$seg6l_endx X:X::X:X$seg6l_endx_nh6|\
-	      End_T$seg6l_endt (1-4294967295)$seg6l_endt_table|\
-	      End_DX4$seg6l_enddx4 A.B.C.D$seg6l_enddx4_nh4|\
-	      End_DX6$seg6l_enddx6 X:X::X:X$seg6l_enddx6_nh6|\
-	      End_DT6$seg6l_enddt6 (1-4294967295)$seg6l_enddt6_table|\
-	      End_DT4$seg6l_enddt4 (1-4294967295)$seg6l_enddt4_table|\
-	      End_DT46$seg6l_enddt46 (1-4294967295)$seg6l_enddt46_table>\
+	  <End_B6_Encap$end_b6_encap|uB6_Encap$ub6_encap>\
+	 nexthop-seg6 X:X::X:X$seg6_nh6\
+	 encap X:X::X:X$seg6_seg1 X:X::X:X$seg6_seg2\
+         [usid-block-length (1-128)$lcblen] [usid-function-length (1-128)$lcfunclen] \
 	  (1-1000000)$routes [repeat (2-1000)$rpt]",
-       "Sharp routing Protocol\n"
-       "install some routes\n"
-       "Routes to install\n"
-       "The vrf we would like to install into if non-default\n"
-       "The NAME of the vrf\n"
-       "v6 Address to start /32 generation at\n"
-       "Nexthop-seg6local to use\n"
-       "Output device to use\n"
-       "SRv6 End function to use\n"
-       "SRv6 End.X function to use\n"
-       "V6 Nexthop address to use\n"
-       "SRv6 End.T function to use\n"
-       "Redirect table id to use\n"
-       "SRv6 End.DX4 function to use\n"
-       "V4 Nexthop address to use\n"
-       "SRv6 End.DX6 function to use\n"
-       "V6 Nexthop address to use\n"
-       "SRv6 End.DT6 function to use\n"
-       "Redirect table id to use\n"
-       "SRv6 End.DT4 function to use\n"
-       "Redirect table id to use\n"
-       "SRv6 End.DT46 function to use\n"
-       "Redirect table id to use\n"
-       "How many to create\n"
-       "Should we repeat this command\n"
-       "How many times to repeat this command\n")
+      "Sharp routing Protocol\n"
+      "install some routes\n"
+      "Routes to install\n"
+      "The vrf we would like to install into if non-default\n"
+      "The NAME of the vrf\n"
+      "v6 Address to start /32 generation at\n"
+      "Nexthop-seg6local to use\n"
+      "Output device to use\n"
+      "SRv6 End.B6.Encap function to use\n"
+      "SRv6 uB6.Encap function to use\n"
+      "Nexthop-seg6 to use\n"
+      "V6 Nexthop address to use\n"
+      "Encap mode\n"
+      "Segment List, 1st SID to use\n"
+      "Segment List, 2nd SID to use\n"
+      "uSID locator block length\n"
+      "Value in bits\n"
+      "uSID node Function length\n"
+      "Value in bits\n"
+      "How many to create\n"
+      "Should we repeat this command\n"
+      "How many times to repeat this command\n")
 {
 	struct vrf *vrf;
 	uint32_t route_flags = 0;
 	struct seg6local_context ctx = {};
 	enum seg6local_action_t action;
+	struct in6_addr seg_list[2];
+	struct in6_addr *p_seg_list = &seg_list[0];
 
 	sg.r.total_routes = routes;
 	sg.r.installed_routes = 0;
@@ -506,6 +499,140 @@ DEFPY (install_seg6local_routes,
 	sg.r.orig_prefix.prefixlen = 128;
 	sg.r.orig_prefix.u.prefix6 = start6;
 
+	if (!vrf_name)
+		vrf_name = VRF_DEFAULT_NAME;
+
+	vrf = vrf_lookup_by_name(vrf_name);
+	if (!vrf) {
+		vty_out(vty, "The vrf NAME specified: %s does not exist\n", vrf_name);
+		return CMD_WARNING;
+	}
+
+	ctx.nh6 = seg6_nh6;
+	action = ZEBRA_SEG6_LOCAL_ACTION_END_B6_ENCAP;
+	if (ub6_encap) {
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
+		if (lcblen)
+			ctx.flv.lcblock_len = lcblen;
+		if (lcfunclen)
+			ctx.flv.lcnode_func_len = lcfunclen;
+	}
+
+	sg.r.nhop.type = NEXTHOP_TYPE_IPV6_IFINDEX;
+	sg.r.nhop.gate.ipv6 = seg6_nh6;
+	sg.r.nhop.ifindex = ifname2ifindex(seg6l_oif, vrf->vrf_id);
+	sg.r.nhop.vrf_id = vrf->vrf_id;
+	sg.r.nhop_group.nexthop = &sg.r.nhop;
+	nexthop_add_srv6_seg6local(&sg.r.nhop, action, &ctx);
+	sg.r.nhop_group.nexthop = &sg.r.nhop;
+	seg_list[0] = seg6_seg1;
+	seg_list[1] = seg6_seg2;
+	nexthop_add_srv6_seg6(&sg.r.nhop, p_seg_list, 2, SRV6_HEADEND_BEHAVIOR_H_ENCAPS);
+
+	sg.r.vrf_id = vrf->vrf_id;
+	sharp_install_routes_helper(&sg.r.orig_prefix, sg.r.vrf_id, sg.r.inst, 0, &sg.r.nhop_group,
+				    &sg.r.backup_nhop_group, routes, route_flags, sg.r.opaque);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY (install_seg6local_routes,
+       install_seg6local_routes_cmd,
+       "sharp install seg6local-routes [vrf NAME$vrf_name]\
+	  X:X::X:X$start6\
+	  nexthop-seg6local NAME$seg6l_oif\
+	     <End$seg6l_end|\
+	      uN$seg6l_micro_end|\
+	      End_X$seg6l_endx X:X::X:X$seg6l_endx_nh6|\
+	      uA$seg6l_micro_endx X:X::X:X$seg6l_micro_endx_nh6|\
+	      End_T$seg6l_endt (1-4294967295)$seg6l_endt_table|\
+	      uDT$seg6l_micro_endt (1-4294967295)$seg6l_micro_endt_table|\
+	      End_DX4$seg6l_enddx4 A.B.C.D$seg6l_enddx4_nh4|\
+	      uDX4$seg6l_micro_enddx4 A.B.C.D$seg6l_micro_enddx4_nh4|\
+	      End_DX6$seg6l_enddx6 X:X::X:X$seg6l_enddx6_nh6|\
+	      uDX6$seg6l_micro_enddx6 X:X::X:X$seg6l_micro_enddx6_nh6|\
+	      End_DT6$seg6l_enddt6 (1-4294967295)$seg6l_enddt6_table|\
+	      uDT6$seg6l_micro_enddt6 (1-4294967295)$seg6l_micro_enddt6_table|\
+	      End_DT4$seg6l_enddt4 (1-4294967295)$seg6l_enddt4_table|\
+	      uDT4$seg6l_micro_enddt4 (1-4294967295)$seg6l_micro_enddt4_table|\
+	      End_DT46$seg6l_enddt46 (1-4294967295)$seg6l_enddt46_table|\
+	      uDT46$seg6l_micro_enddt46 (1-4294967295)$seg6l_micro_enddt46_table>\
+	      [usid-block-length (1-128)$lcblen] [usid-function-length (1-128)$lcfunclen] \
+	      [psp-flavor$psp_flavor] \
+	  (1-1000000)$routes [repeat (2-1000)$rpt]",
+       "Sharp routing Protocol\n"
+       "install some routes\n"
+       "Routes to install\n"
+       "The vrf we would like to install into if non-default\n"
+       "The NAME of the vrf\n"
+       "v6 Address to start /128 generation at\n"
+       "Nexthop-seg6local to use\n"
+       "Output device to use\n"
+       "SRv6 End function to use\n"
+       "SRv6 uN function to use\n"
+       "SRv6 End.X function to use\n"
+       "V6 Nexthop address to use\n"
+       "SRv6 uA function to use\n"
+       "V6 Nexthop address to use\n"
+       "SRv6 End.T function to use\n"
+       "Redirect table id to use\n"
+       "SRv6 uDT function to use\n"
+       "Redirect table id to use\n"
+       "SRv6 End.DX4 function to use\n"
+       "V4 Nexthop address to use\n"
+       "SRv6 uDX4 function to use\n"
+       "V4 Nexthop address to use\n"
+       "SRv6 End.DX6 function to use\n"
+       "V6 Nexthop address to use\n"
+       "SRv6 uDX6 function to use\n"
+       "V6 Nexthop address to use\n"
+       "SRv6 End.DT6 function to use\n"
+       "Redirect table id to use\n"
+       "SRv6 uDT6 function to use\n"
+       "Redirect table id to use\n"
+       "SRv6 End.DT4 function to use\n"
+       "Redirect table id to use\n"
+       "SRv6 uDT4 function to use\n"
+       "Redirect table id to use\n"
+       "SRv6 End.DT46 function to use\n"
+       "Redirect table id to use\n"
+       "SRv6 uDT46 function to use\n"
+       "Redirect table id to use\n"
+       "uSID locator block length\n"
+       "Value in bits\n"
+       "uSID node Function length\n"
+       "Value in bits\n"
+       "Add the PSP flavor\n"
+       "How many to create\n"
+       "Should we repeat this command\n"
+       "How many times to repeat this command\n")
+{
+	struct vrf *vrf;
+	uint32_t route_flags = 0;
+	struct seg6local_context ctx = {};
+	enum seg6local_action_t action;
+
+	sg.r.total_routes = routes;
+	sg.r.installed_routes = 0;
+
+	if (rpt >= 2)
+		sg.r.repeat = rpt * 2;
+	else
+		sg.r.repeat = 0;
+
+	memset(&sg.r.orig_prefix, 0, sizeof(sg.r.orig_prefix));
+	nexthop_del_srv6_seg6local(&sg.r.nhop);
+	nexthop_del_srv6_seg6(&sg.r.nhop);
+	memset(&sg.r.nhop, 0, sizeof(sg.r.nhop));
+	memset(&sg.r.nhop_group, 0, sizeof(sg.r.nhop_group));
+	memset(&sg.r.backup_nhop, 0, sizeof(sg.r.nhop));
+	memset(&sg.r.backup_nhop_group, 0, sizeof(sg.r.nhop_group));
+	sg.r.opaque[0] = '\0';
+	sg.r.inst = 0;
+	sg.r.orig_prefix.family = AF_INET6;
+	sg.r.orig_prefix.prefixlen = IPV6_MAX_BITLEN;
+	sg.r.orig_prefix.u.prefix6 = start6;
+
 	if (!vrf_name)
 		vrf_name = VRF_DEFAULT_NAME;
 
@@ -519,28 +646,69 @@ DEFPY (install_seg6local_routes,
 	if (seg6l_enddx4) {
 		action = ZEBRA_SEG6_LOCAL_ACTION_END_DX4;
 		ctx.nh4 = seg6l_enddx4_nh4;
+	} else if (seg6l_micro_enddx4) {
+		action = ZEBRA_SEG6_LOCAL_ACTION_END_DX4;
+		ctx.nh4 = seg6l_micro_enddx4_nh4;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
 	} else if (seg6l_enddx6) {
 		action = ZEBRA_SEG6_LOCAL_ACTION_END_DX6;
 		ctx.nh6 = seg6l_enddx6_nh6;
+	} else if (seg6l_micro_enddx6) {
+		action = ZEBRA_SEG6_LOCAL_ACTION_END_DX6;
+		ctx.nh6 = seg6l_enddx6_nh6;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
 	} else if (seg6l_endx) {
 		action = ZEBRA_SEG6_LOCAL_ACTION_END_X;
 		ctx.nh6 = seg6l_endx_nh6;
+	} else if (seg6l_micro_endx) {
+		action = ZEBRA_SEG6_LOCAL_ACTION_END_X;
+		ctx.nh6 = seg6l_micro_endx_nh6;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
 	} else if (seg6l_endt) {
 		action = ZEBRA_SEG6_LOCAL_ACTION_END_T;
 		ctx.table = seg6l_endt_table;
+	} else if (seg6l_micro_endt) {
+		action = ZEBRA_SEG6_LOCAL_ACTION_END_T;
+		ctx.table = seg6l_micro_endt_table;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
 	} else if (seg6l_enddt6) {
 		action = ZEBRA_SEG6_LOCAL_ACTION_END_DT6;
 		ctx.table = seg6l_enddt6_table;
+	} else if (seg6l_micro_enddt6) {
+		action = ZEBRA_SEG6_LOCAL_ACTION_END_DT6;
+		ctx.table = seg6l_micro_enddt6_table;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
 	} else if (seg6l_enddt4) {
 		action = ZEBRA_SEG6_LOCAL_ACTION_END_DT4;
 		ctx.table = seg6l_enddt4_table;
+	} else if (seg6l_micro_enddt4) {
+		action = ZEBRA_SEG6_LOCAL_ACTION_END_DT4;
+		ctx.table = seg6l_micro_enddt4_table;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
 	} else if (seg6l_enddt46) {
 		action = ZEBRA_SEG6_LOCAL_ACTION_END_DT46;
 		ctx.table = seg6l_enddt46_table;
+	} else if (seg6l_micro_enddt46) {
+		action = ZEBRA_SEG6_LOCAL_ACTION_END_DT46;
+		ctx.table = seg6l_micro_enddt46_table;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
+	} else if (seg6l_micro_end) {
+		action = ZEBRA_SEG6_LOCAL_ACTION_END;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
+		if (psp_flavor)
+			SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_PSP);
 	} else {
 		action = ZEBRA_SEG6_LOCAL_ACTION_END;
+		if (psp_flavor)
+			SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_PSP);
 	}
 
+	if (CHECK_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID)) {
+		if (lcblen)
+			ctx.flv.lcblock_len = lcblen;
+		if (lcfunclen)
+			ctx.flv.lcnode_func_len = lcfunclen;
+	}
 	sg.r.nhop.type = NEXTHOP_TYPE_IFINDEX;
 	sg.r.nhop.ifindex = ifname2ifindex(seg6l_oif, vrf->vrf_id);
 	sg.r.nhop.vrf_id = vrf->vrf_id;
@@ -945,7 +1113,7 @@ DEFPY (neigh_discover,
 		prefix.u.prefix4 = dst4;
 	} else {
 		prefix.family = AF_INET6;
-		prefix.prefixlen = 128;
+		prefix.prefixlen = IPV6_MAX_BITLEN;
 		prefix.u.prefix6 = dst6;
 	}
 
@@ -1443,12 +1611,55 @@ DEFPY (tc_filter_rate,
 	return CMD_SUCCESS;
 }
 
+/* for testing builtin crash handler & backtrace */
+DEFPY(crashme_segv,
+      crashme_segv_cmd,
+      "sharp crashtest segv",
+      SHARP_STR
+      "crashtest functions\n"
+      "*(int *)1 = 1\n")
+{
+#if defined(__COVERITY__) || defined(__clang_analyzer__)
+	vty_out(vty, "static analysis build, this should not happen?!\n");
+#else
+	/* GCC is too clever, complains if it knows the pointer is just "1" */
+	intptr_t one = atoi("1");
+
+	*(int *)one = 1;
+#endif
+	return CMD_SUCCESS;
+}
+
+/* for testing ASAN & Valgrind warnings */
+DEFPY(crashme_uaf,
+      crashme_uaf_cmd,
+      "sharp crashtest use-after-free",
+      SHARP_STR
+      "crashtest functions\n"
+      "free(p); vty_out(p)\n")
+{
+#if defined(__COVERITY__) || defined(__clang_analyzer__)
+	vty_out(vty, "static analysis build, this should not happen?!\n");
+#else
+	int *p, *f;
+
+	p = f = XCALLOC(MTYPE_TMP, sizeof(*p));
+	*p = 12345;
+	XFREE(MTYPE_TMP, p);
+
+	vty_out(vty, "use-after-free: %d\n", *f);
+#endif
+	return CMD_SUCCESS;
+}
+
+
 void sharp_vty_init(void)
 {
 	install_element(ENABLE_NODE, &install_routes_data_dump_cmd);
 	install_element(ENABLE_NODE, &install_routes_cmd);
 	install_element(ENABLE_NODE, &install_seg6_routes_cmd);
 	install_element(ENABLE_NODE, &install_seg6local_routes_cmd);
+	install_element(ENABLE_NODE, &install_seg6local_segs_routes_cmd);
 	install_element(ENABLE_NODE, &remove_routes_cmd);
 	install_element(ENABLE_NODE, &vrf_label_cmd);
 	install_element(ENABLE_NODE, &sharp_nht_data_dump_cmd);
@@ -1482,5 +1693,7 @@ void sharp_vty_init(void)
 
 	install_element(ENABLE_NODE, &tc_filter_rate_cmd);
 
+	install_element(ENABLE_NODE, &crashme_segv_cmd);
+	install_element(ENABLE_NODE, &crashme_uaf_cmd);
 	return;
 }
