@@ -10,6 +10,8 @@
 
 #if !defined(GNU_LINUX) && !defined(OPEN_BSD)
 
+#include <sys/sysctl.h>
+
 #include "if.h"
 #include "sockunion.h"
 #include "prefix.h"
@@ -128,6 +130,8 @@ void interface_list(struct zebra_ns *zns)
 
 	/* Free sysctl buffer. */
 	XFREE(MTYPE_TMP, ref);
+
+	zebra_dplane_startup_stage(zns, ZEBRA_DPLANE_INTERFACES_READ);
 }
 
 #endif /* !defined(GNU_LINUX) && !defined(OPEN_BSD) */
