@@ -1054,7 +1054,7 @@ void rtm_read(struct rt_msghdr *rtm)
 	/*
 	 * Ignore our own messages.
 	 */
-	if (rtm->rtm_type != RTM_GET && rtm->rtm_pid == pid)
+	if (rtm->rtm_type != RTM_GET && rtm->rtm_pid == zebra_pid)
 		return;
 
 	if (dest.sa.sa_family == AF_INET) {
@@ -1079,12 +1079,10 @@ void rtm_read(struct rt_msghdr *rtm)
 		else
 			p.prefixlen = ip6_masklen(mask.sin6.sin6_addr);
 
-#ifdef KAME
 		if (IN6_IS_ADDR_LINKLOCAL(&gate.sin6.sin6_addr)) {
 			ifindex = IN6_LINKLOCAL_IFINDEX(gate.sin6.sin6_addr);
 			SET_IN6_LINKLOCAL_IFINDEX(gate.sin6.sin6_addr, 0);
 		}
-#endif /* KAME */
 
 		if (!nh.type) {
 			nh.type = ifindex ? NEXTHOP_TYPE_IPV6_IFINDEX
@@ -1468,10 +1466,12 @@ static void routing_socket(struct zebra_ns *zns)
 
 void interface_list_second(struct zebra_ns *zns)
 {
+	zebra_dplane_startup_stage(zns, ZEBRA_DPLANE_ADDRESSES_READ);
 }
 
 void interface_list_tunneldump(struct zebra_ns *zns)
 {
+	zebra_dplane_startup_stage(zns, ZEBRA_DPLANE_TUNNELS_READ);
 }
 
 /* Exported interface function.  This function simply calls
@@ -1625,6 +1625,7 @@ void kernel_update_multi(struct dplane_ctx_list_head *ctx_list)
 		case DPLANE_OP_INTF_ADDR_DEL:
 		case DPLANE_OP_STARTUP_STAGE:
 		case DPLANE_OP_SRV6_ENCAP_SRCADDR_SET:
+		case DPLANE_OP_VLAN_INSTALL:
 			zlog_err("Unhandled dplane data for %s",
 				 dplane_op2str(dplane_ctx_get_op(ctx)));
 			res = ZEBRA_DPLANE_REQUEST_FAILURE;
